<?php
/*
Plugin Name: Laskuhari for WooCommerce
Plugin URI: https://www.laskuhari.fi/woocommerce-laskutus
Description: Lisää automaattilaskutuksen maksutavaksi WooCommerce-verkkokauppaan sekä mahdollistaa tilausten manuaalisen laskuttamisen
Version: 0.9.26
Author: Datahari Solutions
Author URI: https://www.datahari.fi
License: GPLv2
*/

/*
Based on WooCommerce Custom Payment Gateways
Author: Mehdi Akram
Author URI: http://shamokaldarpon.com/
*/

//Additional links on the plugin page
add_filter( 'plugin_row_meta', 'LHPG_register_plugin_links', 10, 2 );
function LHPG_register_plugin_links($links, $file) {
    $base = plugin_basename(__FILE__);
    if ($file == $base) {
        $links[] = '<a href="https://oma.laskuhari.fi/" target="_blank">' . __( 'Kirjaudu Laskuhariin', 'rsb' ) . '</a>';
    }
    return $links;
}

function laskuhari_metabox()
{
    add_meta_box(
        'laskuhari_metabox',           // Unique ID
        'Laskuhari',  // Box title
        'laskuhari_metabox_html',  // Content callback, must be of type callable
        'shop_order',
        'side',
        'core'
    );
}

add_action('add_meta_boxes', 'laskuhari_metabox');

function laskuhari_metabox_html($post)
{
    $laskunumero = get_post_meta($post->ID, '_laskuhari_invoice_number', true);
    $lahetetty = get_post_meta($post->ID, '_laskuhari_sent', true);
    if( $laskunumero > 0 ) {
        $lasku_luotu = true;
        $tila = "LASKU LUOTU";
        $tila_class = " luotu";
        if( $lahetetty ) {
            $tila = "LASKUTETTU";
            $tila_class = " laskutettu";
        }
    } else {
        $lasku_luotu = false;
        $tila = "EI LASKUTETTU";
        $tila_class = " ei-laskutettu";
    }
    ?>
    <div class="laskuhari-tila<?php echo $tila_class; ?>"><?php echo $tila; ?></div>
    <?php
    $edit_link = get_edit_post_link( $post );
    if( $lasku_luotu ) {
        echo '<div class="laskuhari-laskunumero">Lasku '.$laskunumero.'</div>';
        echo '<a class="laskuhari-nappi lataa-lasku" href="'.$edit_link.'&laskuhari_download=current" target="_blank">Lataa PDF</a>';
        echo '<a class="laskuhari-nappi laheta-lasku" href="'.$edit_link.'&laskuhari_send_invoice=current" onclick="if(!confirm(\'Haluatko varmasti lähettää laskun'.($lahetetty ? ' uudelleen' : '').'?\')) return false;">Lähetä lasku'.($lahetetty ? ' uudelleen' : '').'</a>';
        echo '<a class="laskuhari-nappi uusi-lasku" href="'.$edit_link.'&laskuhari=send" onclick="if(!confirm(\'Tämä luo uuden laskun uudella laskunumerolla. Jatketaanko?\')) return false;">Tee uusi lasku</a>';
    } else {
        echo '<a class="laskuhari-nappi laskuta" href="'.$edit_link.'&laskuhari=send" onclick="if(!confirm(\'Haluatko varmasti laskuttaa tämän tilauksen?\')) return false;">Laskuta</a>';
    }
}

add_action( 'woocommerce_cart_calculate_fees','wc_custom_surcharge', 10, 1 );
function wc_custom_surcharge( $cart ) {
    if ( is_admin() && ! defined( 'DOING_AJAX' ) ) {
        return;
    }
    
    if( ! isset( WC()->session->chosen_payment_method ) || WC()->session->chosen_payment_method !== "laskuhari" ) {
        return;
    }
    
    $laskuhari = new WC_Gateway_Laskuhari(true);
    $laskutuslisa = $laskuhari->laskutuslisa;

    if( $laskutuslisa == 0 ) {
        return;
    }
    
    $cart->add_fee( __('Laskutuslisä', 'woocommerce'), $laskutuslisa/1.24, true);
}

/* WooCommerce fallback notice. */
function woocommerce_laskuhari_fallback_notice() {
    echo '<div class="error"><p>' . sprintf( __( 'WooCommerce Laskuhari Payment Gateway depends on the last version of %s to work!', 'LHPG' ), '<a href="http://wordpress.org/extend/plugins/woocommerce/">WooCommerce</a>' ) . '</p></div>';
}

/* Load functions. */
function laskuhari_payment_gateway_load() {
    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        add_action( 'admin_notices', 'woocommerce_laskuhari_fallback_notice' );
        return;
    }
   
    function laskuhari_add_gateway( $methods ) {
        $methods[] = 'WC_Gateway_Laskuhari';
        return $methods;
    }
    add_filter( 'woocommerce_payment_gateways', 'laskuhari_add_gateway' );
    
    
    // Include the WooCommerce Custom Payment Gateways classes.
    require_once plugin_dir_path( __FILE__ ) . 'class-wc-gateway-laskuhari.php';

    function laskuhari_tee_lasku( $order ) {
        laskuhari_process_action($order, true);
    }

    add_action( 'woocommerce_order_action_wc_laskuhari_laskuta', 'laskuhari_tee_lasku' );

    $laskuhari = new WC_Gateway_Laskuhari(true);
    if($laskuhari->get_option('enabled') == 'yes') {
        add_action( 'admin_print_scripts', 'laskuhari_add_scripts' );
        add_action( 'admin_print_styles', 'laskuhari_add_styles' );
        //add_filter( 'post_row_actions', 'laskuhari_laskuta', 10, 2 );
        add_action( 'woocommerce_admin_order_actions_end', 'laskuhari_laskuta' );
        laskuhari_actions();
    }

    /**
    * Add Settings link to the plugin entry in the plugins menu for WC below 2.1
    **/
    if ( version_compare( WOOCOMMERCE_VERSION, "2.1" ) <= 0 ) {

        add_filter('plugin_action_links', 'laskuhari_plugin_action_links', 10, 2);

        function laskuhari_plugin_action_links($links, $file) {
            static $this_plugin;

            if (!$this_plugin) {
                $this_plugin = plugin_basename(__FILE__);
            }

            if ($file == $this_plugin) {
                $settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wc-settings&tab=checkout&section=laskuhari">Asetukset</a>';
                array_unshift($links, $settings_link);
            }
            return $links;
        }
    }
    /**
    * Add Settings link to the plugin entry in the plugins menu for WC 2.1 and above
    **/
    else{
        add_filter('plugin_action_links', 'laskuhari_plugin_action_links', 10, 2);

        function laskuhari_plugin_action_links($links, $file) {
            static $this_plugin;

            if (!$this_plugin) {
                $this_plugin = plugin_basename(__FILE__);
            }

            if ($file == $this_plugin) {
                $settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wc-settings&tab=checkout&section=laskuhari">Asetukset</a>';
                array_unshift($links, $settings_link);
            }
            return $links;
        }
    }
}

add_action( 'woocommerce_after_register_post_type', 'laskuhari_payment_gateway_load', 0 );

function laskuhari_actions() {
    if( isset($_GET['action']) && $_GET['action'] == "laskuhari-batch-send") {
        $order_ids = array();
        $notices   = array();
        $successes = array();

        foreach( $_GET['post'] as $order_id ) {
            $lh = laskuhari_process_action($order_id, true);
            $order      = $lh['order'];
            $notice     = $lh['notice'];
            $success    = $lh['success'];

            $order_ids[] = $order->get_id();
            $notices[]   = urlencode($notice);
            $successes[] = urlencode($success);
        }

        $back = remove_query_arg(array('_wpnonce', 'order_id', 'laskuhari', 'post', 'action'));
        $back = add_query_arg(array('laskuhari_lahetetty' => $order_ids, 'laskuhari_notice' => $notices, 'laskuhari_success' => $successes), $back);
        wp_redirect($back);
        exit;
    }

    if( isset($_GET['laskuhari']) && $_GET['laskuhari'] == "send" ) {
        $lh         = laskuhari_process_action($_GET['order_id'] ? $_GET['order_id'] : $_GET['post'], true);
        $order      = $lh['order'];
        $notice     = $lh['notice'];
        $success    = $lh['success'];

        $back = remove_query_arg(array('_wpnonce', 'order_id', 'laskuhari'));
        $back = add_query_arg(array('laskuhari_lahetetty' => $order->get_id(), 'laskuhari_notice' => urlencode($notice), 'laskuhari_success' => urlencode($success)), $back);
        wp_redirect($back);
        exit;
    }

    if( isset($_GET['laskuhari_download']) ) {
        if( $_GET['laskuhari_download'] > 0 ) {
            laskuhari_download($_GET['laskuhari_download'], $_GET['order_id']);
        } else if( $_GET['laskuhari_download'] == "current" ) {
            laskuhari_download(false, $_GET['post']);
        }
    }

    if( isset($_GET['laskuhari_send_invoice']) ) {
        $send = laskuhari_send_invoice(wc_get_order($_GET['post']));

        $back = remove_query_arg(array('_wpnonce', 'order_id', 'laskuhari_send_invoice'));
        $back = add_query_arg(array('laskuhari_lahetetty' => $_GET['post'], 'laskuhari_notice' => urlencode($send['notice']), 'laskuhari_success' => urlencode($send['success'])), $back);
        wp_redirect($back);
        exit;
    }
}

if( isset($_GET['laskuhari_lahetetty']) ) {
    add_action( 'admin_notices', 'laskuhari_lahetetty' );
}

function laskuhari_lahetetty() {
    $notices = is_array($_GET['laskuhari_notice']) ? $_GET['laskuhari_notice'] : array($_GET['laskuhari_notice']);
    $successes = is_array($_GET['laskuhari_success']) ? $_GET['laskuhari_success'] : array($_GET['laskuhari_success']);
    $order_ids = is_array($_GET['laskuhari_lahetetty']) ? $_GET['laskuhari_lahetetty'] : array($_GET['laskuhari_lahetetty']);
    foreach( $notices as $key => $notice ) {
        if($notice != "") {
            echo '<div class="notice notice-error is-dismissible"><p>'.htmlspecialchars($_GET['laskuhari_notice']).'</p></div>';
        } else {
            $success = $successes[$key] ? $successes[$key] : 'Tilaus #'.$order_ids[$key].' laskutettu.';
            echo '<div class="notice notice-success is-dismissible"><p>'.htmlspecialchars($success).'</p></div>'; 
        }
    }
}
function laskuhari_add_styles() {
    wp_enqueue_style(
        'laskuhari-css',
        plugins_url('css/staili.css' , __FILE__),
        array()
    );
}
function laskuhari_add_scripts() {
    wp_enqueue_script(
        'laskuhari-js',
        plugins_url('js/js.js' , __FILE__),
        array('jquery')
    );
}

function laskuhari_invoice_number_by_order( $orderid ) {
    return get_post_meta($orderid, '_laskuhari_invoice_number', true);
}

    
function laskuhari_laskuta($order){    
    
    if ($order->post->post_type=='shop_order') {
        
        $url = admin_url( 'edit.php?post_type=shop_order&order_id=' . $order->get_id() );

        $sent = get_post_meta($order->get_id(), '_laskuhari_sent', true);
        $invoice_number = get_post_meta($order->get_id(), '_laskuhari_invoice_number', true);
        if($sent == "yes") {
            $uud = " uudelleen?";
            if(!$invoice_number) {
                $uud .= " Lasku luodaan uutena laskuna.";
            }
            $uud2 = " uudelleen";
            $img = "_sent";
        } else {
            $uud = "?";
            $uud2 = "";
            $img = "";
        }
        
        $send_create_invoice_link = wp_nonce_url( add_query_arg( array( 'laskuhari' => 'send', 'laskuhari_invoice_number' => $invoice_number ), $url ), 'edit_order_nonce' );
        $download_invoice_link = wp_nonce_url( add_query_arg( array( 'laskuhari_download' => $invoice_number ), $url ), 'edit_order_nonce' );
        
        // show buttons only on processing and completed orders
        if ( $order->get_status() != 'processing' && $order->get_status() != 'completed' ) {
            return;
        }
        ?>
        <a onclick="if(!confirm('Haluatko varmasti lähettää laskun tilauksesta<?php echo $uud; ?>')) {return false;} else {laskuhari_loading();}" href="<?php echo $send_create_invoice_link; ?>" class="button tips laskuhari-invoice" alt="Lähetä lasku asiakkaalle<?php echo $uud2; ?>" data-tip="Lähetä lasku asiakkaalle<?php echo $uud2; ?>">
            <img src="<?php echo plugins_url( 'images/invoice'.$img.'.png' , __FILE__ ); ?>" alt="Lähetä lasku asiakkaalle<?php echo $uud2; ?>" width="32">
        </a>
        <?php
        if($invoice_number > 0) {
            ?>
            <a href="<?php echo $download_invoice_link; ?>" target="_blank" class="button tips laskuhari-invoice" alt="Lataa laskun PDF-tiedosto" data-tip="Lataa laskun PDF-tiedosto">
                <img src="<?php echo plugins_url( 'images/invoice_download.png' , __FILE__ ); ?>" alt="Lataa laskun PDF-tiedosto" width="32">
            </a>
            <?php
        }


    }
            
}

function laskuhari_download($invoice_number, $order_id) {
    // laskunlähetyksen asetukset
    $info = new WC_Gateway_Laskuhari(true);
    $laskuhari_uid          = $info->uid;
    $laskuhari_apikey       = $info->apikey;

    if( $invoice_number === false ) {
        $invoice_number = laskuhari_invoice_number_by_order( $order_id );
    }

    // luodaan vahvistuskoodi rajapintaa varten
    $t              = time();
    $digest_src     = $laskuhari_uid."+".$t."+".$laskuhari_apikey;
    $dt = hash("sha256", $digest_src);

    // laskunluontirajapinnan URL
    $url         =  "https://oma.laskuhari.fi/api/invoice/".$invoice_number."?uid=".$laskuhari_uid."&t=".$t."&dt=".$dt;
    $post        =  "action=getpdf";
    // suoritetaan curl
    $ch       = laskuhari_curl($post, $url);
    $response = curl_exec($ch);

    // tarkastetaan virheet
    if(curl_errno($ch) || stripos(".".$response, "error")) {
        update_post_meta($order_id, '_laskuhari_invoice_number', '');
        add_action( 'admin_notices', 'laskuhari_getpdf_fail' );
        return;
    }

    //ohjataan PDF-tiedostoon jos ei ollut virheitä
    wp_redirect($response);
    exit;
}

function laskuhari_getpdf_fail() {
    echo '<div class="notice notice-error is-dismissible"><p>Tilauksen PDF-laskun lataaminen epäonnistui</p></div>';
}

function laskuhari_curl($post, $url) {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_FAILONERROR, 1);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
    curl_setopt($ch, CURLOPT_TIMEOUT, 100);
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $post);

    // laskunlähetyksen asetukset
    $info = new WC_Gateway_Laskuhari(true);
    if( $info->enforce_ssl != "yes" ) {
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    }
    
    return $ch;
}

function laskuhari_rivi( $i, $data ) {
    return  "&laskurivi[".$i."][tyyppi]=0".
            "&laskurivi[".$i."][koodi]=".
            "&laskurivi[".$i."][nimike]=".urlencode($data['nimike']).
            "&laskurivi[".$i."][maara]=".urlencode($data['maara']).
            "&laskurivi[".$i."][yks]=".
            "&laskurivi[".$i."][veroton]=".urlencode($data['veroton']).
            "&laskurivi[".$i."][alv]=".urlencode($data['alv']).
            "&laskurivi[".$i."][verollinen]=".$yks_verollinen.
            "&laskurivi[".$i."][ale]=".urlencode($data['ale']).
            "&laskurivi[".$i."][yhtveroton]=".urlencode($data['yhtveroton']).
            "&laskurivi[".$i."][yhtverollinen]=".urlencode($data['yhtverollinen']).
            "&laskurivi[".$i."][toistovali]=0".
            "&laskurivi[".$i."][ennakko]=0".
            "&laskurivi[".$i."][jatkuvuus]=0".
            "&laskurivi[".$i."][automaatio]=0".
            "&laskurivi[".$i."][toistoale]=0";
}

function laskuhari_process_action($order_id, $_send_laskuhari, $order_object = false) {
    global $wc_order_types;
    
    $error_notice = "";
    $success = "";

    if(is_a($order_object, WC_Order::class)) {
        $order = $order_object;
    } else {
        $order = wc_get_order($order_id);
    }
    
    // laskunlähetyksen asetukset
    $info = new WC_Gateway_Laskuhari(true);
    $laskuhari_uid          = $info->uid;
    $laskuhari_apikey       = $info->apikey;
    $laskutuslisa           = $info->laskutuslisa;

    // tilaajan tiedot
    $customer               = $order->get_address('billing');
    $shippingdata           = $order->get_address('shipping');

    $shipping_different = false;
    foreach ($customer as $key => $cdata) {
        if($shippingdata[$key] != $cdata && isset($shippingdata[$key])) {
            $shipping_different = true;
            break;
        }
    }
    if(!$shipping_different) {
        foreach($shippingdata as $key => $sdata) {
            $shippingdata[$key] = "";
        }
    }

    // tilatut tuotteet
    $products           = $order->get_items();

    // summat
    $loppusumma         = $order->get_total();
    $toimitustapa       = utf8_decode($order->get_shipping_method());
    $toimitusmaksu      = $order->get_total_shipping();
    $toimitus_vero      = $order->get_shipping_tax();
    $alennus            = $order->get_total_discount();
    $cart_discount      = $order->get_discount_total();
    $cart_discount_tax  = $order->get_discount_tax();

    // luodaan vahvistuskoodi rajapintaa varten
    $t          = time();
    $digest_src = $laskuhari_uid."+".$t."+".$laskuhari_apikey;
    $dt         = hash("sha256", $digest_src);

    // laskunluontirajapinnan URL
    $url         =  "https://oma.laskuhari.fi/api/invoice?uid=".$laskuhari_uid."&t=".$t."&dt=".$dt;
    $post        =  "action=create&ref=wc";

    $post       .=  '&maksuehto='.
                    '&pvm='.date('d.m.Y').
                    '&lahetetty=0'.
                    '&viitteenne='.urlencode("Tilausnro. ".$order->get_order_number()).
                    '&yritys='.urlencode($customer['company']).
                    '&henkilo='.urlencode($customer['first_name'].' '.$customer['last_name']).
                    '&lahiosoite='.urlencode(trim($customer['address_1'].' '.$customer['address_2'])).
                    '&postinumero='.urlencode($customer['postcode']).
                    '&postitoimipaikka='.urlencode($customer['city']).
                    '&toimitus_yritys='.urlencode($shippingdata['company']).
                    '&toimitus_henkilo='.urlencode($shippingdata['first_name'].' '.$shippingdata['last_name']).
                    '&toimitus_lahiosoite='.urlencode(trim($shippingdata['address_1'].' '.$shippingdata['address_2'])).
                    '&toimitus_postinumero='.urlencode($shippingdata['postcode']).
                    '&toimitus_postitoimipaikka='.urlencode($shippingdata['city']).
                    '&email='.urlencode($customer['email']).
                    '&ytunnus='.
                    '&puhelin='.urlencode($customer['phone']);

    $i = 1;
    $laskettu_summa = 0;
    foreach($products as $item) {
        
        $data = $item->get_data();

        $yht_verollinen = round($data['subtotal'] + $data['subtotal_tax'], 2);

        if( $data['subtotal'] != 0 ) {
            $alv         = round($data['subtotal_tax']/$data['subtotal']*100, 0);
            $yht_veroton = $yht_verollinen / (1+$alv/100);
        } else {
            $alv         = 0;
            $yht_veroton = 0;
        }

        if( $data['subtotal'] != 0 ) {
            $yks_verollinen = round($yht_verollinen/$data['quantity'], 2);
            $yks_veroton    = $yks_verollinen/(1+$alv/100);
        } else {
            $yks_verollinen = 0;
            $yks_veroton    = 0;
        }

        $ale = 0;

        $post .= laskuhari_rivi($i, [
            "nimike"        => utf8_decode($data['name']),
            "maara"         => $data['quantity'],
            "veroton"       => $yks_veroton,
            "alv"           => $alv,
            "verollinen"    => $yks_verollinen,
            "ale"           => $ale,
            "yhtveroton"    => $yht_veroton,
            "yhtverollinen" => $yht_verollinen
        ]);
        
        $i++;
        $laskettu_summa += $yht_verollinen;
    }


    // lisätään toimitusmaksu
    if($toimitusmaksu > 0) {
        $post .= laskuhari_rivi($i, [
            "nimike"        => "Toimitustapa: ".$toimitustapa,
            "maara"         => 1,
            "veroton"       => $toimitusmaksu,
            "alv"           => round($toimitus_vero/$toimitusmaksu*100, 0),
            "verollinen"    => ($toimitusmaksu+$toimitus_vero),
            "ale"           => 0,
            "yhtveroton"    => $toimitusmaksu,
            "yhtverollinen" => ($toimitusmaksu+$toimitus_vero)
        ]);

        $i++;
        $laskettu_summa += ($toimitusmaksu+$toimitus_vero);
    }

    // lisätään alennus
    if($cart_discount) {
        $post .= laskuhari_rivi($i, [
            "nimike"        => "Alennus",
            "maara"         => 1,
            "veroton"       => ($cart_discount*-1),
            "alv"           => round($cart_discount_tax/$cart_discount*100, 0),
            "verollinen"    => (($cart_discount+$cart_discount_tax)*-1),
            "ale"           => 0,
            "yhtveroton"    => ($cart_discount*-1),
            "yhtverollinen" => (($cart_discount+$cart_discount_tax)*-1)
        ]);

        $i++;
        $laskettu_summa += (($cart_discount+$cart_discount_tax)*-1);
    }

    // lisätään laskutuslisä
    if($laskutuslisa) {
        $post .= laskuhari_rivi($i, [
            "nimike"        => "Laskutuslisä",
            "maara"         => 1,
            "veroton"       => $laskutuslisa/(1+$alv/100),
            "alv"           => $alv,
            "verollinen"    => $laskutuslisa,
            "ale"           => 0,
            "yhtveroton"    => $laskutuslisa/(1+$alv/100),
            "yhtverollinen" => $laskutuslisa
        ]);
        $laskettu_summa += $laskutuslisa;
        $i++;
    }

    if(round($loppusumma, 2) != round($laskettu_summa, 2)) {
        $post .= laskuhari_rivi($i, [
            "nimike"        => "Desimaalikorjaus",
            "maara"         => 1,
            "veroton"       => ($loppusumma-$laskettu_summa),
            "alv"           => 0,
            "verollinen"    => ($loppusumma-$laskettu_summa),
            "ale"           => 0,
            "yhtveroton"    => ($loppusumma-$laskettu_summa),
            "yhtverollinen" => ($loppusumma-$laskettu_summa)
        ]);
    }

    // suoritetaan curl
    if( ! $_GET['laskuhari_invoice_number'] ) {
        $ch       = laskuhari_curl($post, $url);
        $response = curl_exec($ch);

        // tarkastetaan virheet
        if(curl_errno($ch)) {
            $order->update_status( 'processing', $error_notice='Virhe laskun lähetyksessä. cURL errno. '.curl_errno().": ".curl_error() );
            if(function_exists('wc_add_notice')) wc_add_notice( 'Laskun automaattinen lähetys epäonnistui. Lähetämme laskun manuaalisesti.', 'notice' );
        }
        if(stripos(".".$response, "error")) {
            $order->update_status( 'processing', $error_notice='Virhe laskun lähetyksessä: '.$response );
            if(function_exists('wc_add_notice')) wc_add_notice( 'Laskun automaattinen lähetys epäonnistui. Lähetämme laskun manuaalisesti.', 'notice' );
        }

        $response = json_decode($response, true);
        $laskunro = $response['new_invoice_number'];

        curl_close($ch);
    } else {
        $laskunro = $_GET['laskuhari_invoice_number'];
    }

    // jatketaan vain, jos ei ollut virheitä
    if($error_notice == "") {

        update_post_meta( $order->get_id(), '_laskuhari_invoice_number', $laskunro );
        
        $order->add_order_note( __( 'Lasku #'.$laskunro.' luotu Laskuhariin', 'laskuhari' ) );

        // LASKUN LÄHETYS

        laskuhari_send_invoice( $order, $_send_laskuhari );

    } else {
        $order->add_order_note( __( 'Laskun luominen Laskuhariin epäonnistui', 'laskuhari' ) );
    }

    return array(
        "order" => $order,
        "notice" => $error_notice,
        "success" => $success
    );
}

function laskuhari_send_invoice($order, $_send_laskuhari = false) {
    // laskunlähetyksen asetukset
    $info = new WC_Gateway_Laskuhari(true);
    $laskuhari_uid          = $info->uid;
    $laskuhari_apikey       = $info->apikey;
    $sendername             = $info->laskuttaja;
    $email_message          = $info->laskuviesti;
    $laskutuslisa           = $info->laskutuslisa;

    $laskunro = get_post_meta($order->get_id(), '_laskuhari_invoice_number', true);

    if($_send_laskuhari == true) {
       $lahetystapa = $info->lahetystapa_manuaalinen;
    } else {
       $lahetystapa = $info->lahetystapa;
    }

    if(trim(rtrim($email_message)) == "") {
        $email_message = "Liitteenä lasku.";
    }

    // tilaajan tiedot
    $customer               = $order->get_address('billing');

    // luodaan POST-pyyntö
    $post = "action=send&ref=wc";
    if($customer['email'] != "") {
        $sendername      = $sendername ? $sendername : "Laskutus";
        $post           .= "&invoiceType=email";
        $post           .= "&eMailSenderName=".$sendername;
        $post           .= "&receiverEmail=".urlencode($customer['email']);
        $post           .= "&eMailSubject=Lasku";
        $post           .= "&eMailMessage=".urlencode($email_message);
        $kirjelasku      = false;
    } else {
        $kirjelasku = true;
        $post .= "&invoiceType=letter";
    }

    if(
        ($lahetystapa == "kirje" && $kirjelasku == true) ||
        ($lahetystapa == "email" && $kirjelasku == false) ||
        ($lahetystapa == "auto")            
    ) {

        // luodaan vahvistuskoodi rajapintaa varten
        $t          = time();
        $digest_src = $laskuhari_uid."+".$t."+".$laskuhari_apikey;
        $dt         = hash("sha256", $digest_src);

        // laskunlähetysrajapinnan URL
        $url        = "https://oma.laskuhari.fi/api/invoice/".($laskunro*1)."?uid=".$laskuhari_uid."&t=".$t."&dt=".$dt;

        // suoritetaan curl
        $ch         = laskuhari_curl($post, $url);
        $response   = curl_exec($ch);

        // tarkastetaan virheet
        if(curl_errno($ch)) {
            $order->update_status( 'processing', $error_notice='Virhe laskun lähetyksessä. cURL errno. '.curl_errno().": ".curl_error() );
            if(function_exists('wc_add_notice')) wc_add_notice( 'Laskun automaattinen lähetys epäonnistui. Lähetämme laskun manuaalisesti.', 'notice' );
        }
        if(stripos(".".$response, "error")) {
            $order->update_status( 'processing', $error_notice='Virhe laskun lähetyksessä: '.$response );
            if(function_exists('wc_add_notice')) wc_add_notice( 'Laskun automaattinen lähetys epäonnistui. Lähetämme laskun manuaalisesti.', 'notice' );
        }

        curl_close($ch);

        $response = json_decode($response, true);

        if($error_notice == "") {
            $order->update_status( 'processing', 'Lasku lähetetty.' );
            update_post_meta( $order->get_id(), '_laskuhari_sent', 'yes' );
        }
    } else {
        if($_send_laskuhari == true) {
            $success = "Tilauksen #".$order->get_id()." lasku tallennettu Laskuhariin. Laskua ei lähetetty vielä.";
        }
        $order->update_status( 'processing', 'Odottaa laskutusta' );
    }

    return array(
        "order" => $order,
        "notice" => $error_notice,
        "success" => $success
    );

}

?>